/***************************************************
//Web: http://www.buydisplay.com
EastRising Technology Co.,LTD
****************************************************/
#include <stdlib.h>
#include "ER-EPD0213-1B.h"
#include "Debug.h"
#include "GUI_Paint.h"


/******************************************************************************
function :	Software reset
parameter:
******************************************************************************/
void EPD2IN13::EPD_Reset(void)
{
    EPD_RST_1;
    Dev->Dev_Delay_ms(200);
    EPD_RST_0;
    Dev->Dev_Delay_ms(200);
    EPD_RST_1;
    Dev->Dev_Delay_ms(200);
}

/******************************************************************************
function :	send command
parameter:
    Reg : Command register
******************************************************************************/
void EPD2IN13::EPD_SendCommand(UBYTE Reg)
{
    EPD_DC_0;
    EPD_CS_0;
    SPI_Write_Byte(Reg);
    EPD_CS_1;
}

/******************************************************************************
function :	send data
parameter:
    Data : Write data
******************************************************************************/
void EPD2IN13::EPD_SendData(UBYTE Data)
{
    EPD_DC_1;
    EPD_CS_0;
    SPI_Write_Byte(Data);
    EPD_CS_1;
}

/******************************************************************************
function :	Wait until the busy_pin goes LOW
parameter:
******************************************************************************/
void EPD2IN13::EPD_WaitUntilIdle(void)
{
    while(EPD_BUSY_RD == HIGH) {      //LOW: idle, HIGH: busy
        Dev->Dev_Delay_ms(100);
    }
}

/******************************************************************************
function :	Setting the display window
parameter:
******************************************************************************/
void EPD2IN13::EPD_SetWindows(UWORD Xstart, UWORD Ystart, UWORD Xend, UWORD Yend)
{
    EPD_SendCommand(SET_RAM_X_ADDRESS_START_END_POSITION);
    EPD_SendData((Xstart >> 3) & 0xFF);
    EPD_SendData((Xend >> 3) & 0xFF);

    EPD_SendCommand(SET_RAM_Y_ADDRESS_START_END_POSITION);
    EPD_SendData(Ystart & 0xFF);
    EPD_SendData((Ystart >> 8) & 0xFF);
    EPD_SendData(Yend & 0xFF);
    EPD_SendData((Yend >> 8) & 0xFF);
}

/******************************************************************************
function :	Set cursor
parameter:
******************************************************************************/
void EPD2IN13::EPD_SetCursor(UWORD Xstart, UWORD Ystart)
{
    EPD_SendCommand(SET_RAM_X_ADDRESS_COUNTER);
    EPD_SendData((Xstart >> 3) & 0xFF);

    EPD_SendCommand(SET_RAM_Y_ADDRESS_COUNTER);
    EPD_SendData(Ystart & 0xFF);
    EPD_SendData((Ystart >> 8) & 0xFF);
    
    EPD_WaitUntilIdle();
}

/******************************************************************************
function :	Turn On Display
parameter:
******************************************************************************/
void EPD2IN13::EPD_TurnOnDisplay(void)
{
    EPD_SendCommand(DISPLAY_UPDATE_CONTROL_2);
    EPD_SendData(0xC7);
    EPD_SendCommand(MASTER_ACTIVATION);
    EPD_WaitUntilIdle();
}

/******************************************************************************
function :	Initialize the e-Paper register
parameter:
******************************************************************************/
UBYTE EPD2IN13::EPD_Init()
{
    Dev->System_Init();

    EPD_Reset();

    EPD_SendCommand(0x74);  //Set Analog Block Control
    EPD_SendData(0x54);
    EPD_SendCommand(0x7E);  //Set Digital Block Control
    EPD_SendData(0x3B);
    EPD_SendCommand(0x2B);  // Reduce glitch under ACVCOM	
    EPD_SendData(0x04);           
    EPD_SendData(0x63);
 
    EPD_SendCommand(BOOSTER_SOFT_START_CONTROL);		  // Booster Soft start Control
    EPD_SendData(0x8B);		   
    EPD_SendData(0x9C);		   
    EPD_SendData(0x96);			   
    EPD_SendData(0x0F);
       
    EPD_SendCommand(DRIVER_OUTPUT_CONTROL);  // Driver Output control
    EPD_SendData(0xD3);           
    EPD_SendData(0x00);
    EPD_SendData(0x00);     
        
    EPD_SendCommand(DATA_ENTRY_MODE_SETTING);  // Data Entry mode setting
    EPD_SendData(0x03);         
    EPD_SendCommand(SET_RAM_X_ADDRESS_START_END_POSITION); //Set RAM X - address Start / End position
    EPD_SendData(0x00); // RAM x address start at 0
    EPD_SendData(0x0C); //RAM x address end at 0Ch(12+1)*8->104
    EPD_SendCommand(SET_RAM_Y_ADDRESS_START_END_POSITION); //Set Ram Y- address Start / End position
    EPD_SendData(0xD3); // RAM y address start at 0D3h;  
    EPD_SendData(0x00);
    EPD_SendData(0x00); // RAM y address end at 00h;
    EPD_SendData(0x00);
        
        
    EPD_SendCommand(BORDER_WAVEFORM_CONTROL); // Border Waveform Control
    EPD_SendData(0x01); // HIZ

    EPD_SendCommand(DISPLAY_UPDATE_CONTROL_1);
    EPD_SendData(0x00);//Normal
        
 //       unsigned char temp1,temp2;	
    EPD_SendCommand(0x18);//Temperature Sensor Control
    EPD_SendData(0x80);  //Internal temperature sensor
    EPD_SendCommand(DISPLAY_UPDATE_CONTROL_2);//Display UpdateControl 2
    EPD_SendData(0xB1);	//Load Temperature and waveform setting.
    EPD_SendCommand(MASTER_ACTIVATION); //Master Activation
    EPD_WaitUntilIdle();	
    return 0;
}

/******************************************************************************
function :	Clear screen
parameter:
******************************************************************************/
void EPD2IN13::EPD_Clear(void) 
{
    UWORD Width, Height;
    Width = (EPD_WIDTH % 8 == 0)? (EPD_WIDTH / 8 ): (EPD_WIDTH / 8 + 1);
    Height = EPD_HEIGHT;
    
    EPD_SetWindows(0, 0, EPD_WIDTH, EPD_HEIGHT);
    for (UWORD j = 0; j < Height; j++) {
        EPD_SetCursor(0, j);
        EPD_SendCommand(WRITE_RAM);
        for (UWORD i = 0; i < Width; i++) {
            // SendData(color);
            EPD_SendData(0XFF);
        }
    }
   // EPD_TurnOnDisplay();
}

/******************************************************************************
function :	Sends the image buffer in RAM to e-Paper and displays
parameter:
******************************************************************************/
void EPD2IN13::EPD_Display(void)
{
    UWORD Width, Height;
    Width = (EPD_WIDTH % 8 == 0)? (EPD_WIDTH / 8 ): (EPD_WIDTH / 8 + 1);
    Height = EPD_HEIGHT;

    EPD_SetWindows(0, 0, EPD_WIDTH, EPD_HEIGHT);
    for (UWORD j = 0; j < Height; j++) {
        EPD_SetCursor(0, j);
        EPD_SendCommand(WRITE_RAM);
        for (UWORD i = 0; i < Width; i++) {
            EPD_SendData(spiram->SPIRAM_RD_Byte(i + j * Width));
        }
    }
    EPD_TurnOnDisplay();
}

/******************************************************************************
function :	Enter sleep mode
parameter:
******************************************************************************/
void EPD2IN13::EPD_Sleep(void) 
{
    EPD_SendCommand(DEEP_SLEEP_MODE);
    EPD_SendData(0x01);
    EPD_WaitUntilIdle();
}
